package org.open.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;

import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.TrustStrategy;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import org.apache.http.Consts;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.conn.ssl.SSLConnectionSocketFactory;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

/**
 * 用于模拟HTTP请求中GET/POST方式 
 */
public class HttpUtil {  
	
    /** 
     * 发送GET请求 
     * @param url 目的地址 
     * @param parameters 请求参数，Map类型。 
     * @return 远程响应结果 
     */  
    public static String sendGet(String url) { 
        String result = "";
        BufferedReader in = null;// 读取响应输入流  
        try {
            String full_url = url ;
            // 创建URL对象  
            java.net.URL connURL = new java.net.URL(full_url);  
            // 打开URL连接  
            java.net.HttpURLConnection httpConn = (java.net.HttpURLConnection) connURL.openConnection();  
            // 设置通用属性  
            httpConn.setRequestProperty("Accept", "*/*");  
            httpConn.setRequestProperty("Connection", "Keep-Alive");  
            httpConn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)");  
            // 建立实际的连接  
            httpConn.connect();  
            // 响应头部获取  
            //Map<String, List<String>> headers = httpConn.getHeaderFields();  
            // 遍历所有的响应头字段  
            //for (String key : headers.keySet()) {  
            //    System.out.println(key + "\t：\t" + headers.get(key));  
            //}  
            // 定义BufferedReader输入流来读取URL的响应,并设置编码方式  
            in = new BufferedReader(new InputStreamReader(httpConn.getInputStream(), "UTF-8"));  
            String line;  
            // 读取返回的内容  
            while ((line = in.readLine()) != null) {  
                result += line;  
            }  
        } catch (Exception e) {
            e.printStackTrace();
        }finally{
            try {  
                if (in != null) {  
                    in.close();  
                }  
            } catch (IOException ex) {  
                ex.printStackTrace();  
            }  
        }
        return convertJson(result);
    }  
    
    /** 
     * 发送GET请求 
     * @param url 目的地址 
     * @param parameters 请求参数，Map类型。 
     * @return 远程响应结果 
     */  
    public static String sendGet(String url, Map<String, String> param) { 
        // 创建默认的httpClient实例.
        CloseableHttpClient httpclient = HttpClients.createDefault();
        try {
            URIBuilder uriBuilder = new URIBuilder(url);
            if (param != null) {
                for (String key : param.keySet()) {
                    uriBuilder.addParameter(key, param.get(key));
                }
            }
            URI uri = uriBuilder.build();
            HttpGet httpget = new HttpGet(uri);
            RequestConfig requestConfig = RequestConfig.custom()  
                       .setConnectTimeout(3000).setConnectionRequestTimeout(3000)  
                       .setSocketTimeout(5000).build();  
            httpget.setConfig(requestConfig);
            // 执行get请求
            CloseableHttpResponse response = httpclient.execute(httpget);
            try {
                if (response.getStatusLine().getStatusCode() == 200) {
                    org.apache.http.HttpEntity entity = response.getEntity();
                    if (entity != null) {
                        String msg = EntityUtils.toString(entity, "UTF-8");
                        return convertJson(msg);
                    }
                } else {
                    System.out.println("状态码：" + response.getStatusLine().getStatusCode());
                }
            } finally {
                response.close();
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                httpclient.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return null;
    }  
    
    
    private static List<NameValuePair> createParam(Map<String, Object> param) {
        //建立一个NameValuePair数组，用于存储欲传送的参数
        List<NameValuePair> nvps = new ArrayList <NameValuePair>();
        if(param != null) {
            for(String k : param.keySet()) {
                if(param.get(k)!=null) nvps.add(new BasicNameValuePair(k, param.get(k).toString()));
            }
        }
        return nvps;
    }
	
	//请求超时时间,这个时间定义了socket读数据的超时时间，也就是连接到服务器之后到从服务器获取响应数据需要等待的时间,发生超时，会抛出SocketTimeoutException异常。
	private static final int SOCKET_TIME_OUT = 60000;
	//连接超时时间,这个时间定义了通过网络与服务器建立连接的超时时间，也就是取得了连接池中的某个连接之后到接通目标url的连接等待时间。发生超时，会抛出ConnectionTimeoutException异常
	private static final int CONNECT_TIME_OUT = 60000;
	    
	public static String sendPost(String url, Map<String, Object> param) throws Exception{
	    //目前HttpClient最新版的实现类为CloseableHttpClient
		CloseableHttpClient client = HttpClients.createDefault();
		CloseableHttpResponse response = null;
		HttpEntity entity=null;
		try {
		    if(param != null) {
		        //建立Request的对象，一般用目标url来构造，Request一般配置addHeader、setEntity、setConfig
		        HttpPost req = new HttpPost(url);
		        entity=new UrlEncodedFormEntity(createParam(param), Consts.UTF_8);
		        //setHeader,添加头文件
		        req.setHeader("Accept", "*/*");
		        req.setHeader("Connection", "Keep-Alive");
		        req.setHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)");
		        //setConfig,添加配置,如设置请求超时时间,连接超时时间
		        RequestConfig reqConfig = RequestConfig.custom().setSocketTimeout(SOCKET_TIME_OUT).setConnectTimeout(CONNECT_TIME_OUT).build();
		        req.setConfig(reqConfig);
		        //setEntity,添加内容
		        req.setEntity(entity);
		        //执行Request请求,CloseableHttpClient的execute方法返回的response都是CloseableHttpResponse类型
		        //其常用方法有getFirstHeader(String)、getLastHeader(String)、headerIterator（String）取得某个Header name对应的迭代器、getAllHeaders()、getEntity、getStatus等
		        response = client.execute(req);
		        entity =  response.getEntity();
		        //用EntityUtils.toString()这个静态方法将HttpEntity转换成字符串,防止服务器返回的数据带有中文,所以在转换的时候将字符集指定成utf-8就可以了
		        String result= EntityUtils.toString(entity, "UTF-8");
		        if(response.getStatusLine().getStatusCode()==200){
		            return convertJson(result);
		        }else{
		            return null;
		        }
		    }
		} catch(Exception e) {
		   e.printStackTrace();
		}finally{
		    //一定要记得把entity fully consume掉，否则连接池中的connection就会一直处于占用状态
		    EntityUtils.consume(entity);
	    }
		return null;
	}	
	
  
    /** 
     * 发送POST请求 
     * @param url 目的地址 
     * @param parameters 请求参数，Map类型。 
     * @return 远程响应结果 
     */  
    public static String sendPost(String url) {  
        String result = "";// 返回的结果  
        BufferedReader in = null;// 读取响应输入流  
        PrintWriter out = null;  
        StringBuffer sb = new StringBuffer();// 处理请求参数  
        String params = "";// 编码之后的参数  
        try {  
            // 创建URL对象  
            java.net.URL connURL = new java.net.URL(url);  
            // 打开URL连接  
            java.net.HttpURLConnection httpConn = (java.net.HttpURLConnection) connURL.openConnection();  
            // 设置通用属性  
            httpConn.setRequestProperty("Accept", "*/*");  
            httpConn.setRequestProperty("Connection", "Keep-Alive");  
            httpConn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1)");  
            httpConn.setRequestProperty("Charsert", "UTF-8"); 
            // 设置POST方式  
            httpConn.setDoInput(true);  
            httpConn.setDoOutput(true);  
            // 获取HttpURLConnection对象对应的输出流  
            out = new PrintWriter(httpConn.getOutputStream());  
            // 发送请求参数  
            out.write(params);  
            // flush输出流的缓冲  
            out.flush();  
            // 定义BufferedReader输入流来读取URL的响应，设置编码方式  
            in = new BufferedReader(new InputStreamReader(httpConn.getInputStream(), "UTF-8"));  
            String line;  
            // 读取返回的内容  
            while ((line = in.readLine()) != null) {  
                result += line;  
            }  
        } catch (Exception e) {  
            e.printStackTrace();  
        } finally {  
            try {  
                if (out != null) {  
                    out.close();  
                }  
                if (in != null) {  
                    in.close();  
                }  
            } catch (IOException ex) {  
                ex.printStackTrace();  
            }  
        }  
        return convertJson(result);  
    }  
    
    public static String convertJson(String msg) {
        // 以下代码为了json转换正确
        if(msg.startsWith("\"")){
            String msg1 = msg.substring(1);
            String msg2 = msg1.substring(0,msg1.length()-1);
            return msg2.replaceAll("\\\\","");
        } else {
            return msg;
        }
    }
  
    /** 
     * 发送POST请求(Https) 
     * @param url 目的地址 
     * @param parameters 请求参数，Map类型。 
     * @return 远程响应结果 
     */  
    public static String sendHttpsPost(String url, Map<String, Object> param) {
        CloseableHttpClient httpClient = createSSLClientDefault();
        CloseableHttpResponse response = null;
        String result = null;
        HttpEntity entity = null;
        try {
            //创建Post请求，
            HttpPost httpPost = new HttpPost(url);
            entity = new UrlEncodedFormEntity(createParam(param), Consts.UTF_8);
            httpPost.setHeader( "Content-type", "application/x-www-form-urlencoded; charset=utf-8" );
            httpPost.setHeader( "Connection", "keep-alive" );
            httpPost.setEntity(entity);
            //执行Post请求，
            response = httpClient.execute(httpPost);
            //得到响应体
            HttpEntity resEntity = response.getEntity();
            //获取响应内容
            result = EntityUtils.toString(resEntity,"UTF-8");
            result = convertJson(result);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            //一定要记得把entity fully consume掉，否则连接池中的connection就会一直处于占用状态
            try {
                EntityUtils.consume(entity);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    private static CloseableHttpClient createSSLClientDefault() {
        try {
            SSLContext sslContext = new SSLContextBuilder().loadTrustMaterial(null, new TrustStrategy() {
                //信任所有
                public boolean isTrusted(X509Certificate[] chain, String authType) throws CertificateException {
                    return true;
                }
            }).build();
            SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, new HostnameVerifier() {
                @Override
                public boolean verify(String s, SSLSession sslSession) {
                    return true;
                }
            });
            return HttpClients.custom().setSSLSocketFactory(sslsf).build();
        } catch (KeyManagementException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (KeyStoreException e) {
            e.printStackTrace();
        }
        return HttpClients.createDefault();
    }
    
}
