package org.open.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.UUID;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;

/**
 * 数字签名生成工具类
 */
public class SignUtil {
	private static final String DEFAULT_CHARSET = "UTF-8";
	
	/**
	 * 生成随机数
	 */
	public static String getNonce() {
        return UUID.randomUUID().toString();
    }
    
	/**
	 * 生成时间戳
	 * @return
	 */
	public static String getTimeStamp() {
		return System.currentTimeMillis() / 1000 + "";
	}
	
	/**
	 * 生成签名
	 * @param appSecret 申报应用平台获取
	 * @param params 请求数据参数集合
	 */
	public static String createSign(String appSecret, Map<String, Object> params) {
		// 生成签名前先去除sign
		params.remove("sign");
		String stringSignTemp = packageSign(params, true);
		stringSignTemp = stringSignTemp + appSecret;
		return DigestUtils.md5Hex(getContentBytes(stringSignTemp, DEFAULT_CHARSET));
	}
	
	/**
	 * 组装签名的字段
	 * @param params 参数
	 * @return String
	 */
	public static String packageSign(Map<String, Object> params, boolean urlEncoder) {
		// 先将参数以其参数名的字典序升序进行排序
		TreeMap<String, Object> sortedParams = new TreeMap<String, Object>(params);
		// 遍历排序后的字典，将所有参数按"key=value"格式拼接在一起
		StringBuilder sb = new StringBuilder();
		boolean first = true;
		for (Entry<String, Object> param : sortedParams.entrySet()) {
			Object obj = param.getValue();
			String value = "";
			if (obj instanceof String) {
				value = (String)obj;
			} else if (obj instanceof JSONObject) {
				value = ((JSONObject) obj).toJSONString();
			} else if (obj instanceof JSONArray) {
				value = ((JSONArray) obj).toJSONString();
			} else if (obj instanceof Integer) {
			    Integer tmp = (Integer)obj;
			    value =  (tmp==null? "": tmp.toString());
			} else if (obj instanceof Double) {
			    Double tmp = (Double)obj;
	            value =  (tmp==null? "": tmp.toString());
			} else if (obj instanceof Boolean) {
			    Boolean tmp = (Boolean)obj;
                value =  (tmp==null? "": tmp.toString());
            }
			 
			if (StringUtils.isBlank(value)) {
				continue;
			}
			if (first) {
				first = false;
			} else {
				sb.append("&");
			}
			sb.append(param.getKey()).append("=");
			if (urlEncoder) {
				try {
					value = urlEncode(value);
				} catch (UnsupportedEncodingException e) {
				}
			}
			sb.append(value);
		}
		return sb.toString();
	}
	
	public static String urlEncode(String src) throws UnsupportedEncodingException {
		return URLEncoder.encode(src, DEFAULT_CHARSET).replace("+", "%20");
	}
	
    private static byte[] getContentBytes(String content, String charset) {
        if (charset == null || "".equals(charset)) {
            return content.getBytes();
        }
        try {
            return content.getBytes(charset);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("MD5签名过程中出现错误,指定的编码集不对,您目前指定的编码集是:" + charset);
        }
    }
}
